@@ -67,6 +67,7 @@ typedef struct reply_t {
     char *errorposition;
 } reply_t;
 
+static int exit_code = 0;
 static reply_t last_reply;
 
 static int reply_boolean_cb(void *params, int val) {
@@ -76,8 +77,8 @@ static int reply_boolean_cb(void *params, int val) {
 }
 
 static int reply_string_cb(void *params, const unsigned char *val, size_t len) {
-    char *str = scalloc(len + 1, 1);
-    strncpy(str, (const char *)val, len);
+    char *str = sstrndup((const char *)val, len);
+
     if (strcmp(last_key, "error") == 0)
         last_reply.error = str;
     else if (strcmp(last_key, "input") == 0)
@@ -100,14 +101,14 @@ static int reply_end_map_cb(void *params) {
             fprintf(stderr, "ERROR:               %s\n", last_reply.errorposition);
         }
         fprintf(stderr, "ERROR: %s\n", last_reply.error);
+        exit_code = 2;
     }
     return 1;
 }
 
 static int reply_map_key_cb(void *params, const unsigned char *keyVal, size_t keyLen) {
     free(last_key);
-    last_key = scalloc(keyLen + 1, 1);
-    strncpy(last_key, (const char *)keyVal, keyLen);
+    last_key = sstrndup((const char *)keyVal, keyLen);
     return 1;
 }
 
@@ -126,8 +127,7 @@ static yajl_callbacks reply_callbacks = {
 static char *config_last_key = NULL;
 
 static int config_string_cb(void *params, const unsigned char *val, size_t len) {
-    char *str = scalloc(len + 1, 1);
-    strncpy(str, (const char *)val, len);
+    char *str = sstrndup((const char *)val, len);
     if (strcmp(config_last_key, "config") == 0) {
         fprintf(stdout, "%s", str);
     }
@@ -144,8 +144,7 @@ static int config_end_map_cb(void *params) {
 }
 
 static int config_map_key_cb(void *params, const unsigned char *keyVal, size_t keyLen) {
-    config_last_key = scalloc(keyLen + 1, 1);
-    strncpy(config_last_key, (const char *)keyVal, keyLen);
+    config_last_key = sstrndup((const char *)keyVal, keyLen);
     return 1;
 }
 
@@ -326,5 +325,5 @@ int main(int argc, char *argv[]) {
 
     close(sockfd);
 
-    return 0;
+    return exit_code;
 }
