@@ -31,7 +31,7 @@ typedef struct {
      */
     int stop_signal;
     /**
-     * The signal requested by the client to inform it of theun hidden state of i3bar
+     * The signal requested by the client to inform it of the unhidden state of i3bar
      */
     int cont_signal;
 
@@ -85,4 +85,4 @@ bool child_want_click_events(void);
  * Generates a click event, if enabled.
  *
  */
-void send_block_clicked(int button, const char *name, const char *instance, int x, int y);
+void send_block_clicked(int button, const char *name, const char *instance, int x, int y, int x_rel, int y_rel, int width, int height, int mods);
