@@ -213,7 +213,7 @@ static uint32_t predict_statusline_length(bool use_short_text) {
 
         render->width = predict_text_width(text);
         if (block->border)
-            render->width += logical_px(2);
+            render->width += logical_px(block->border_left + block->border_right);
 
         /* Compute offset and append for text aligment in min_width. */
         if (block->min_width <= render->width) {
@@ -287,8 +287,8 @@ static void draw_statusline(i3_output *output, uint32_t clip_left, bool use_focu
 
         color_t bg_color = bar_color;
 
-        int border_width = (block->border) ? logical_px(1) : 0;
         int full_render_width = render->width + render->x_offset + render->x_append;
+        int has_border = block->border ? 1 : 0;
         if (block->border || block->background || block->urgent) {
             /* Let's determine the colors first. */
             color_t border_color = bar_color;
@@ -310,15 +310,16 @@ static void draw_statusline(i3_output *output, uint32_t clip_left, bool use_focu
 
             /* Draw the background. */
             draw_util_rectangle(&output->statusline_buffer, bg_color,
-                                x + border_width,
-                                logical_px(1) + border_width,
-                                full_render_width - 2 * border_width,
-                                bar_height - 2 * border_width - logical_px(2));
+                                x + has_border * logical_px(block->border_left),
+                                logical_px(1) + has_border * logical_px(block->border_top),
+                                full_render_width - has_border * logical_px(block->border_right + block->border_left),
+                                bar_height - has_border * logical_px(block->border_bottom + block->border_top) - logical_px(2));
         }
 
         draw_util_text(text, &output->statusline_buffer, fg_color, bg_color,
-                       x + render->x_offset + border_width, logical_px(ws_voff_px),
-                       render->width - 2 * border_width);
+                       x + render->x_offset + has_border * logical_px(block->border_left),
+                       bar_height / 2 - font.height / 2,
+                       render->width - has_border * logical_px(block->border_left + block->border_right));
         x += full_render_width;
 
         /* If this is not the last block, draw a separator. */
@@ -454,6 +455,50 @@ static bool execute_custom_command(xcb_keycode_t input_code, bool event_is_relea
     return false;
 }
 
+static void child_handle_button(xcb_button_press_event_t *event, i3_output *output, uint32_t statusline_x) {
+    if (statusline_x > (uint32_t)output->statusline_width) {
+        return;
+    }
+
+    /* x of the start of the current block relative to the statusline. */
+    uint32_t last_block_x = 0;
+    struct status_block *block;
+    TAILQ_FOREACH(block, &statusline_head, blocks) {
+        i3String *text;
+        struct status_block_render_desc *render;
+        if (output->statusline_short_text && block->short_text != NULL) {
+            text = block->short_text;
+            render = &block->short_render;
+        } else {
+            text = block->full_text;
+            render = &block->full_render;
+        }
+
+        if (i3string_get_num_bytes(text) == 0) {
+            continue;
+        }
+
+        /* Include the whole block in our calculations: when min_width is
+         * specified, we have to take padding width into account. */
+        const uint32_t full_render_width = render->width + render->x_offset + render->x_append;
+        /* x of the click event relative to the current block. */
+        const uint32_t relative_x = statusline_x - last_block_x;
+        if (relative_x <= full_render_width) {
+            send_block_clicked(event->detail, block->name, block->instance,
+                               event->root_x, event->root_y, relative_x,
+                               event->event_y, full_render_width, bar_height,
+                               event->state);
+            return;
+        }
+
+        last_block_x += full_render_width + block->sep_block_width;
+        if (last_block_x > statusline_x) {
+            /* Click was on a separator. */
+            return;
+        }
+    }
+}
+
 /*
  * Handle a button press event (i.e. a mouse click on one of our bars).
  * We determine, whether the click occurred on a workspace button or if the scroll-
@@ -479,10 +524,6 @@ static void handle_button(xcb_button_press_event_t *event) {
 
     /* During button release events, only check for custom commands. */
     const bool event_is_release = (event->response_type & ~0x80) == XCB_BUTTON_RELEASE;
-    if (event_is_release) {
-        execute_custom_command(event->detail, event_is_release);
-        return;
-    }
 
     int32_t x = event->event_x >= 0 ? event->event_x : 0;
     int workspace_width = 0;
@@ -499,44 +540,32 @@ static void handle_button(xcb_button_press_event_t *event) {
             workspace_width += logical_px(ws_spacing_px);
     }
 
-    if (x > workspace_width && child_want_click_events()) {
-        /* If the child asked for click events,
-         * check if a status block has been clicked. */
-        int tray_width = get_tray_width(walk->trayclients);
-        int last_block_x = 0;
-        int offset = walk->rect.w - walk->statusline_width - tray_width - logical_px((tray_width > 0) * sb_hoff_px);
-        int32_t statusline_x = x - offset;
+    if (child_want_click_events() && x > workspace_width) {
+        const int tray_width = get_tray_width(walk->trayclients);
+        /* Calculate the horizontal coordinate (x) of the start of the
+         * statusline by subtracting its width and the width of the tray from
+         * the bar width. */
+        const int offset = walk->rect.w - walk->statusline_width -
+                           tray_width - logical_px((tray_width > 0) * sb_hoff_px);
+        if (x >= offset) {
+            /* Click was after the start of the statusline, return to avoid
+             * executing any other actions even if a click event is not
+             * produced eventually. */
 
-        if (statusline_x >= 0 && statusline_x < walk->statusline_width) {
-            struct status_block *block;
-
-            TAILQ_FOREACH(block, &statusline_head, blocks) {
-                i3String *text = block->full_text;
-                struct status_block_render_desc *render = &block->full_render;
-                if (walk->statusline_short_text && block->short_text != NULL) {
-                    text = block->short_text;
-                    render = &block->short_render;
-                }
-
-                if (i3string_get_num_bytes(text) == 0)
-                    continue;
-
-                const int relative_x = statusline_x - last_block_x;
-                if (relative_x >= 0 && (uint32_t)relative_x <= render->width) {
-                    send_block_clicked(event->detail, block->name, block->instance,
-                                       event->root_x, event->root_y, relative_x, event->event_y, render->width, bar_height,
-                                       event->state);
-                    return;
-                }
-
-                last_block_x += render->width + render->x_append + render->x_offset + block->sep_block_width;
+            if (!event_is_release) {
+                /* x of the click event relative to the start of the
+                 * statusline. */
+                const uint32_t statusline_x = x - offset;
+                child_handle_button(event, walk, statusline_x);
             }
+
+            return;
         }
     }
 
     /* If a custom command was specified for this mouse button, it overrides
      * the default behavior. */
-    if (execute_custom_command(event->detail, event_is_release)) {
+    if (execute_custom_command(event->detail, event_is_release) || event_is_release) {
         return;
     }
 
@@ -1169,7 +1198,21 @@ char *init_xcb_early(void) {
 
     depth = root_screen->root_depth;
     colormap = root_screen->default_colormap;
-    visual_type = get_visualtype(root_screen);
+    visual_type = config.transparency ? xcb_aux_find_visual_by_attrs(root_screen, -1, 32) : NULL;
+    if (visual_type != NULL) {
+        depth = xcb_aux_get_depth_of_visual(root_screen, visual_type->visual_id);
+        colormap = xcb_generate_id(xcb_connection);
+        xcb_void_cookie_t cm_cookie = xcb_create_colormap_checked(xcb_connection,
+                                                                  XCB_COLORMAP_ALLOC_NONE,
+                                                                  colormap,
+                                                                  xcb_root,
+                                                                  visual_type->visual_id);
+        if (xcb_request_failed(cm_cookie, "Could not allocate colormap")) {
+            exit(EXIT_FAILURE);
+        }
+    } else {
+        visual_type = get_visualtype(root_screen);
+    }
 
     xcb_cursor_context_t *cursor_ctx;
     if (xcb_cursor_context_new(conn, root_screen, &cursor_ctx) == 0) {
