@@ -14,6 +14,7 @@
 #include <sys/time.h>
 #include <sys/resource.h>
 
+#include <xcb/shape.h>
 #include <xcb/xcb_keysyms.h>
 #include <xcb/xkb.h>
 
@@ -70,7 +71,7 @@ extern uint8_t root_depth;
 extern xcb_visualid_t visual_id;
 extern xcb_colormap_t colormap;
 
-extern bool xcursor_supported, xkb_supported;
+extern bool xcursor_supported, xkb_supported, shape_supported;
 extern xcb_window_t root;
 extern struct ev_loop *main_loop;
 extern bool only_check_config;
