@@ -67,17 +67,17 @@ static void GENERATED_call(const int call_identifier, struct CommandResultIR *re
          case 8:
              result->next_state = INITIAL;
 #ifndef TEST_PARSER
-             cmd_move_window_to_position(&current_match, result, get_string("method"), get_long("coord_x"), get_long("coord_y"));
+             cmd_move_window_to_position(&current_match, result, get_long("coord_x"), get_long("coord_y"));
 #else
-           fprintf(stderr, "cmd_move_window_to_position(%s, %ld, %ld)\n", get_string("method"), get_long("coord_x"), get_long("coord_y"));
+           fprintf(stderr, "cmd_move_window_to_position(%ld, %ld)\n", get_long("coord_x"), get_long("coord_y"));
 #endif
              break;
          case 9:
              result->next_state = INITIAL;
 #ifndef TEST_PARSER
-             cmd_move_window_to_position(&current_match, result, get_string("method"), get_long("coord_x"), get_long("coord_y"));
+             cmd_move_window_to_position(&current_match, result, get_long("coord_x"), get_long("coord_y"));
 #else
-           fprintf(stderr, "cmd_move_window_to_position(%s, %ld, %ld)\n", get_string("method"), get_long("coord_x"), get_long("coord_y"));
+           fprintf(stderr, "cmd_move_window_to_position(%ld, %ld)\n", get_long("coord_x"), get_long("coord_y"));
 #endif
              break;
          case 10:
@@ -283,9 +283,9 @@ static void GENERATED_call(const int call_identifier, struct CommandResultIR *re
          case 35:
              result->next_state = INITIAL;
 #ifndef TEST_PARSER
-             cmd_resize(&current_match, result, get_string("way"), get_string("direction"), get_long("resize_px"), 10);
+             cmd_resize(&current_match, result, get_string("way"), get_string("direction"), get_long("resize_px"), 0);
 #else
-           fprintf(stderr, "cmd_resize(%s, %s, %ld, %d)\n", get_string("way"), get_string("direction"), get_long("resize_px"), 10);
+           fprintf(stderr, "cmd_resize(%s, %s, %ld, %d)\n", get_string("way"), get_string("direction"), get_long("resize_px"), 0);
 #endif
              break;
          case 36:
@@ -299,9 +299,9 @@ static void GENERATED_call(const int call_identifier, struct CommandResultIR *re
          case 37:
              result->next_state = INITIAL;
 #ifndef TEST_PARSER
-             cmd_border(&current_match, result, get_string("border_style"), 2);
+             cmd_border(&current_match, result, get_string("border_style"), -1);
 #else
-           fprintf(stderr, "cmd_border(%s, %d)\n", get_string("border_style"), 2);
+           fprintf(stderr, "cmd_border(%s, %d)\n", get_string("border_style"), -1);
 #endif
              break;
          case 38:
@@ -330,13 +330,21 @@ static void GENERATED_call(const int call_identifier, struct CommandResultIR *re
              break;
          case 41:
              result->next_state = INITIAL;
+#ifndef TEST_PARSER
+             cmd_resize_set(&current_match, result, get_long("width"), get_string("mode_width"), 0, 0);
+#else
+           fprintf(stderr, "cmd_resize_set(%ld, %s, %d, %d)\n", get_long("width"), get_string("mode_width"), 0, 0);
+#endif
+             break;
+         case 42:
+             result->next_state = INITIAL;
 #ifndef TEST_PARSER
              cmd_title_format(&current_match, result, get_string("format"));
 #else
            fprintf(stderr, "cmd_title_format(%s)\n", get_string("format"));
 #endif
              break;
-         case 42:
+         case 43:
              result->next_state = INITIAL;
 #ifndef TEST_PARSER
              cmd_fullscreen(&current_match, result, get_string("action"), "output");
@@ -344,7 +352,7 @@ static void GENERATED_call(const int call_identifier, struct CommandResultIR *re
            fprintf(stderr, "cmd_fullscreen(%s, %s)\n", get_string("action"), "output");
 #endif
              break;
-         case 43:
+         case 44:
              result->next_state = INITIAL;
 #ifndef TEST_PARSER
              cmd_scratchpad_show(&current_match, result);
@@ -352,20 +360,12 @@ static void GENERATED_call(const int call_identifier, struct CommandResultIR *re
            fprintf(stderr, "cmd_scratchpad_show()\n");
 #endif
              break;
-         case 44:
+         case 45:
              result->next_state = INITIAL;
 #ifndef TEST_PARSER
              cmd_resize(&current_match, result, get_string("way"), get_string("direction"), 10, 10);
 #else
            fprintf(stderr, "cmd_resize(%s, %s, %d, %d)\n", get_string("way"), get_string("direction"), 10, 10);
-#endif
-             break;
-         case 45:
-             result->next_state = INITIAL;
-#ifndef TEST_PARSER
-             cmd_workspace(&current_match, result, get_string("direction"));
-#else
-           fprintf(stderr, "cmd_workspace(%s)\n", get_string("direction"));
 #endif
              break;
          case 46:
@@ -394,13 +394,21 @@ static void GENERATED_call(const int call_identifier, struct CommandResultIR *re
              break;
          case 49:
              result->next_state = INITIAL;
+#ifndef TEST_PARSER
+             cmd_workspace(&current_match, result, get_string("direction"));
+#else
+           fprintf(stderr, "cmd_workspace(%s)\n", get_string("direction"));
+#endif
+             break;
+         case 50:
+             result->next_state = INITIAL;
 #ifndef TEST_PARSER
              cmd_workspace_back_and_forth(&current_match, result);
 #else
            fprintf(stderr, "cmd_workspace_back_and_forth()\n");
 #endif
              break;
-         case 50:
+         case 51:
              result->next_state = INITIAL;
 #ifndef TEST_PARSER
              cmd_workspace_name(&current_match, result, get_string("workspace"), get_string("no_auto_back_and_forth"));
@@ -408,20 +416,12 @@ static void GENERATED_call(const int call_identifier, struct CommandResultIR *re
            fprintf(stderr, "cmd_workspace_name(%s, %s)\n", get_string("workspace"), get_string("no_auto_back_and_forth"));
 #endif
              break;
-         case 51:
+         case 52:
              result->next_state = INITIAL;
 #ifndef TEST_PARSER
              cmd_bar(&current_match, result, get_string("bar_type"), get_string("bar_value"), get_string("bar_id"));
 #else
            fprintf(stderr, "cmd_bar(%s, %s, %s)\n", get_string("bar_type"), get_string("bar_value"), get_string("bar_id"));
-#endif
-             break;
-         case 52:
-             result->next_state = CRITERIA;
-#ifndef TEST_PARSER
-             cmd_criteria_add(&current_match, result, get_string("ctype"), NULL);
-#else
-           fprintf(stderr, "cmd_criteria_add(%s, NULL)\n", get_string("ctype"));
 #endif
              break;
          case 53:
@@ -433,19 +433,19 @@ static void GENERATED_call(const int call_identifier, struct CommandResultIR *re
 #endif
              break;
          case 54:
-             result->next_state = INITIAL;
+             result->next_state = CRITERIA;
 #ifndef TEST_PARSER
-             cmd_criteria_match_windows(&current_match, result);
+             cmd_criteria_add(&current_match, result, get_string("ctype"), NULL);
 #else
-           fprintf(stderr, "cmd_criteria_match_windows()\n");
+           fprintf(stderr, "cmd_criteria_add(%s, NULL)\n", get_string("ctype"));
 #endif
              break;
          case 55:
              result->next_state = INITIAL;
 #ifndef TEST_PARSER
-             cmd_debuglog(&current_match, result, get_string("argument"));
+             cmd_criteria_match_windows(&current_match, result);
 #else
-           fprintf(stderr, "cmd_debuglog(%s)\n", get_string("argument"));
+           fprintf(stderr, "cmd_criteria_match_windows()\n");
 #endif
              break;
          case 56:
@@ -467,9 +467,9 @@ static void GENERATED_call(const int call_identifier, struct CommandResultIR *re
          case 58:
              result->next_state = INITIAL;
 #ifndef TEST_PARSER
-             cmd_floating(&current_match, result, get_string("floating"));
+             cmd_debuglog(&current_match, result, get_string("argument"));
 #else
-           fprintf(stderr, "cmd_floating(%s)\n", get_string("floating"));
+           fprintf(stderr, "cmd_debuglog(%s)\n", get_string("argument"));
 #endif
              break;
          case 59:
@@ -489,6 +489,14 @@ static void GENERATED_call(const int call_identifier, struct CommandResultIR *re
 #endif
              break;
          case 61:
+             result->next_state = INITIAL;
+#ifndef TEST_PARSER
+             cmd_floating(&current_match, result, get_string("floating"));
+#else
+           fprintf(stderr, "cmd_floating(%s)\n", get_string("floating"));
+#endif
+             break;
+         case 62:
              result->next_state = CRITERIA;
 #ifndef TEST_PARSER
              cmd_criteria_init(&current_match, result);
@@ -496,7 +504,7 @@ static void GENERATED_call(const int call_identifier, struct CommandResultIR *re
            fprintf(stderr, "cmd_criteria_init()\n");
 #endif
              break;
-         case 62:
+         case 63:
              result->next_state = INITIAL;
 #ifndef TEST_PARSER
              cmd_exit(&current_match, result);
@@ -504,7 +512,7 @@ static void GENERATED_call(const int call_identifier, struct CommandResultIR *re
            fprintf(stderr, "cmd_exit()\n");
 #endif
              break;
-         case 63:
+         case 64:
              result->next_state = INITIAL;
 #ifndef TEST_PARSER
              cmd_restart(&current_match, result);
@@ -512,7 +520,7 @@ static void GENERATED_call(const int call_identifier, struct CommandResultIR *re
            fprintf(stderr, "cmd_restart()\n");
 #endif
              break;
-         case 64:
+         case 65:
              result->next_state = INITIAL;
 #ifndef TEST_PARSER
              cmd_reload(&current_match, result);
@@ -520,20 +528,12 @@ static void GENERATED_call(const int call_identifier, struct CommandResultIR *re
            fprintf(stderr, "cmd_reload()\n");
 #endif
              break;
-         case 65:
+         case 66:
              result->next_state = INITIAL;
 #ifndef TEST_PARSER
              cmd_open(&current_match, result);
 #else
            fprintf(stderr, "cmd_open()\n");
-#endif
-             break;
-         case 66:
-             result->next_state = INITIAL;
-#ifndef TEST_PARSER
-             cmd_border(&current_match, result, get_string("border_style"), 0);
-#else
-           fprintf(stderr, "cmd_border(%s, %d)\n", get_string("border_style"), 0);
 #endif
              break;
          case 67:
@@ -547,9 +547,9 @@ static void GENERATED_call(const int call_identifier, struct CommandResultIR *re
          case 68:
              result->next_state = INITIAL;
 #ifndef TEST_PARSER
-             cmd_border(&current_match, result, get_string("border_style"), 1);
+             cmd_border(&current_match, result, "pixel", 1);
 #else
-           fprintf(stderr, "cmd_border(%s, %d)\n", get_string("border_style"), 1);
+           fprintf(stderr, "cmd_border(%s, %d)\n", "pixel", 1);
 #endif
              break;
          case 69:
