@@ -19,9 +19,9 @@ typedef enum {
     BAR_COLORS_BACKGROUND = 17,
     BAR_WORKSPACE_BUTTONS = 18,
     DEFAULT_BORDER_PIXELS = 19,
-    BAR_SEPARATOR_SYMBOL = 20,
-    FORCE_FOCUS_WRAPPING = 21,
-    WORKSPACE_OUTPUT_STR = 22,
+    WORKSPACE_OUTPUT_WORD = 20,
+    BAR_SEPARATOR_SYMBOL = 21,
+    FORCE_FOCUS_WRAPPING = 22,
     BAR_BINDSYM_COMMAND = 23,
     BAR_TRAY_PADDING_PX = 24,
     DEFAULT_ORIENTATION = 25,
