@@ -63,14 +63,15 @@ static cmdp_token tokens_DEFAULT_BORDER_PIXELS[2] = {
     { "end", "", __CALL, { 14 } },
     { "number", "width", DEFAULT_BORDER_PIXELS_PX, { 0 } },
 };
+static cmdp_token tokens_WORKSPACE_OUTPUT_WORD[2] = {
+    { "word", "output", __CALL, { 15 } },
+    { "end", "", INITIAL, { 0 } },
+};
 static cmdp_token tokens_BAR_SEPARATOR_SYMBOL[1] = {
-    { "string", "separator", __CALL, { 15 } },
+    { "string", "separator", __CALL, { 16 } },
 };
 static cmdp_token tokens_FORCE_FOCUS_WRAPPING[1] = {
-    { "word", "value", __CALL, { 16 } },
-};
-static cmdp_token tokens_WORKSPACE_OUTPUT_STR[1] = {
-    { "string", "output", __CALL, { 17 } },
+    { "word", "value", __CALL, { 17 } },
 };
 static cmdp_token tokens_BAR_BINDSYM_COMMAND[2] = {
     { "'--release", "release", BAR_BINDSYM_COMMAND, { 0 } },
@@ -176,7 +177,7 @@ static cmdp_token tokens_WORKSPACE_LAYOUT[4] = {
     { "'tabbed", "layout", __CALL, { 51 } },
 };
 static cmdp_token tokens_WORKSPACE_OUTPUT[1] = {
-    { "'output", "", WORKSPACE_OUTPUT_STR, { 0 } },
+    { "'output", "", WORKSPACE_OUTPUT_WORD, { 0 } },
 };
 static cmdp_token tokens_BAR_BAR_COMMAND[1] = {
     { "string", "command", __CALL, { 52 } },
@@ -539,9 +540,9 @@ static cmdp_token_ptr tokens[95] = {
     { tokens_BAR_COLORS_BACKGROUND, 1 },
     { tokens_BAR_WORKSPACE_BUTTONS, 1 },
     { tokens_DEFAULT_BORDER_PIXELS, 2 },
+    { tokens_WORKSPACE_OUTPUT_WORD, 2 },
     { tokens_BAR_SEPARATOR_SYMBOL, 1 },
     { tokens_FORCE_FOCUS_WRAPPING, 1 },
-    { tokens_WORKSPACE_OUTPUT_STR, 1 },
     { tokens_BAR_BINDSYM_COMMAND, 2 },
     { tokens_BAR_TRAY_PADDING_PX, 2 },
     { tokens_DEFAULT_ORIENTATION, 3 },
