@@ -87,31 +87,30 @@ void match_copy(Match *dest, Match *src) {
 bool match_matches_window(Match *match, i3Window *window) {
     LOG("Checking window 0x%08x (class %s)\n", window->id, window->class_class);
 
-    if (match->class != NULL) {
-        if (window->class_class == NULL)
-            return false;
-        if (strcmp(match->class->pattern, "__focused__") == 0 &&
-            strcmp(window->class_class, focused->window->class_class) == 0) {
-            LOG("window class matches focused window\n");
-        } else if (regex_matches(match->class, window->class_class)) {
-            LOG("window class matches (%s)\n", window->class_class);
-        } else {
-            return false;
-        }
-    }
+#define GET_FIELD_str(field) (field)
+#define GET_FIELD_i3string(field) (i3string_as_utf8(field))
+#define CHECK_WINDOW_FIELD(match_field, window_field, type)                                       \
+    do {                                                                                          \
+        if (match->match_field != NULL) {                                                         \
+            if (window->window_field == NULL) {                                                   \
+                return false;                                                                     \
+            }                                                                                     \
+                                                                                                  \
+            const char *window_field_str = GET_FIELD_##type(window->window_field);                \
+            if (strcmp(match->match_field->pattern, "__focused__") == 0 &&                        \
+                focused && focused->window && focused->window->window_field &&                    \
+                strcmp(window_field_str, GET_FIELD_##type(focused->window->window_field)) == 0) { \
+                LOG("window " #match_field " matches focused window\n");                          \
+            } else if (regex_matches(match->match_field, window_field_str)) {                     \
+                LOG("window " #match_field " matches (%s)\n", window_field_str);                  \
+            } else {                                                                              \
+                return false;                                                                     \
+            }                                                                                     \
+        }                                                                                         \
+    } while (0)
 
-    if (match->instance != NULL) {
-        if (window->class_instance == NULL)
-            return false;
-        if (strcmp(match->instance->pattern, "__focused__") == 0 &&
-            strcmp(window->class_instance, focused->window->class_instance) == 0) {
-            LOG("window instance matches focused window\n");
-        } else if (regex_matches(match->instance, window->class_instance)) {
-            LOG("window instance matches (%s)\n", window->class_instance);
-        } else {
-            return false;
-        }
-    }
+    CHECK_WINDOW_FIELD(class, class_class, str);
+    CHECK_WINDOW_FIELD(instance, class_instance, str);
 
     if (match->id != XCB_NONE) {
         if (window->id == match->id) {
@@ -122,33 +121,8 @@ bool match_matches_window(Match *match, i3Window *window) {
         }
     }
 
-    if (match->title != NULL) {
-        if (window->name == NULL)
-            return false;
-
-        const char *title = i3string_as_utf8(window->name);
-        if (strcmp(match->title->pattern, "__focused__") == 0 &&
-            strcmp(title, i3string_as_utf8(focused->window->name)) == 0) {
-            LOG("window title matches focused window\n");
-        } else if (regex_matches(match->title, title)) {
-            LOG("title matches (%s)\n", title);
-        } else {
-            return false;
-        }
-    }
-
-    if (match->window_role != NULL) {
-        if (window->role == NULL)
-            return false;
-        if (strcmp(match->window_role->pattern, "__focused__") == 0 &&
-            strcmp(window->role, focused->window->role) == 0) {
-            LOG("window role matches focused window\n");
-        } else if (regex_matches(match->window_role, window->role)) {
-            LOG("window_role matches (%s)\n", window->role);
-        } else {
-            return false;
-        }
-    }
+    CHECK_WINDOW_FIELD(title, name, i3string);
+    CHECK_WINDOW_FIELD(window_role, role, str);
 
     if (match->window_type != UINT32_MAX) {
         if (window->window_type == match->window_type) {
