@@ -9,21 +9,112 @@
  */
 #include "all.h"
 
-typedef enum { BEFORE,
-               AFTER } position_t;
+/*
+ * Returns the lowest container in the tree that has both a and b as descendants.
+ *
+ */
+static Con *lowest_common_ancestor(Con *a, Con *b) {
+    Con *parent_a = a;
+    while (parent_a) {
+        Con *parent_b = b;
+        while (parent_b) {
+            if (parent_a == parent_b) {
+                return parent_a;
+            }
+            parent_b = parent_b->parent;
+        }
+        parent_a = parent_a->parent;
+    }
+    assert(false);
+}
+
+/*
+ * Returns the direct child of ancestor that contains con.
+ *
+ */
+static Con *child_containing_con_recursively(Con *ancestor, Con *con) {
+    Con *child = con;
+    while (child && child->parent != ancestor) {
+        child = child->parent;
+        assert(child->parent);
+    }
+    return child;
+}
+
+/*
+ * Returns true if the given container is the focused descendant of ancestor, recursively.
+ *
+ */
+static bool is_focused_descendant(Con *con, Con *ancestor) {
+    Con *current = con;
+    while (current != ancestor) {
+        if (TAILQ_FIRST(&(current->parent->focus_head)) != current) {
+            return false;
+        }
+        current = current->parent;
+        assert(current->parent);
+    }
+    return true;
+}
 
 /*
  * This function detaches 'con' from its parent and inserts it either before or
  * after 'target'.
  *
  */
-static void insert_con_into(Con *con, Con *target, position_t position) {
+void insert_con_into(Con *con, Con *target, position_t position) {
     Con *parent = target->parent;
     /* We need to preserve the old con->parent. While it might still be used to
      * insert the entry before/after it, we call the on_remove_child callback
      * afterwards which might then close the con if it is empty. */
     Con *old_parent = con->parent;
 
+    /* We compare the focus order of the children of the lowest common ancestor. If con or
+     * its ancestor is before target's ancestor then con should be placed before the target
+     * in the focus stack. */
+    Con *lca = lowest_common_ancestor(con, parent);
+    if (lca == con) {
+        ELOG("Container is being inserted into one of its descendants.\n");
+        return;
+    }
+
+    Con *con_ancestor = child_containing_con_recursively(lca, con);
+    Con *target_ancestor = child_containing_con_recursively(lca, target);
+    bool moves_focus_from_ancestor = is_focused_descendant(con, con_ancestor);
+    bool focus_before;
+
+    /* Determine if con is going to be placed before or after target in the parent's focus stack. */
+    if (con_ancestor == target_ancestor) {
+        /* Happens when the target is con's old parent. Eg with layout V [ A H [ B C ] ],
+         * if we move C up. Target will be H. */
+        focus_before = moves_focus_from_ancestor;
+    } else {
+        /* Look at the focus stack order of the children of the lowest common ancestor. */
+        Con *current;
+        TAILQ_FOREACH(current, &(lca->focus_head), focused) {
+            if (current == con_ancestor || current == target_ancestor) {
+                break;
+            }
+        }
+        focus_before = (current == con_ancestor);
+    }
+
+    /* If con is the focused container in our old ancestor we place the new ancestor
+     * before the old ancestor in the focus stack. Example:
+     * Consider the layout [ H [ V1 [ A* B ] V2 [ C ] ] ] where A is focused. We move to
+     * a second workspace and from there we move A to the right and switch back to the
+     * original workspace. Without the change focus would move to B instead of staying
+     * with A. */
+    if (moves_focus_from_ancestor && focus_before) {
+        Con *place = TAILQ_PREV(con_ancestor, focus_head, focused);
+        TAILQ_REMOVE(&(lca->focus_head), target_ancestor, focused);
+        if (place) {
+            TAILQ_INSERT_AFTER(&(lca->focus_head), place, target_ancestor, focused);
+        } else {
+            TAILQ_INSERT_HEAD(&(lca->focus_head), target_ancestor, focused);
+        }
+    }
+
     con_detach(con);
     con_fix_percent(con->parent);
 
@@ -46,12 +137,28 @@ static void insert_con_into(Con *con, Con *target, position_t position) {
 
     con->parent = parent;
 
+    if (parent == lca) {
+        if (focus_before) {
+            /* Example layout: H [ A B* ], we move A up/down. 'target' will be H. */
+            TAILQ_INSERT_BEFORE(target, con, focused);
+        } else {
+            /* Example layout: H [ A B* ], we move A up/down. 'target' will be H. */
+            TAILQ_INSERT_AFTER(&(parent->focus_head), target, con, focused);
+        }
+    } else {
+        if (focus_before) {
+            /* Example layout: V [ H [ A B ] C* ], we move C up. 'target' will be A. */
+            TAILQ_INSERT_HEAD(&(parent->focus_head), con, focused);
+        } else {
+            /* Example layout: V [ H [ A* B ] C ], we move C up. 'target' will be A. */
+            TAILQ_INSERT_TAIL(&(parent->focus_head), con, focused);
+        }
+    }
+
     if (position == BEFORE) {
         TAILQ_INSERT_BEFORE(target, con, nodes);
-        TAILQ_INSERT_HEAD(&(parent->focus_head), con, focused);
     } else if (position == AFTER) {
         TAILQ_INSERT_AFTER(&(parent->nodes_head), target, con, nodes);
-        TAILQ_INSERT_HEAD(&(parent->focus_head), con, focused);
     }
 
     /* Pretend the con was just opened with regards to size percent values.
@@ -71,25 +178,24 @@ static void insert_con_into(Con *con, Con *target, position_t position) {
  */
 static void attach_to_workspace(Con *con, Con *ws, direction_t direction) {
     con_detach(con);
-    con_fix_percent(con->parent);
-
-    CALL(con->parent, on_remove_child);
-
+    Con *old_parent = con->parent;
     con->parent = ws;
 
     if (direction == D_RIGHT || direction == D_DOWN) {
         TAILQ_INSERT_HEAD(&(ws->nodes_head), con, nodes);
-        TAILQ_INSERT_HEAD(&(ws->focus_head), con, focused);
     } else {
         TAILQ_INSERT_TAIL(&(ws->nodes_head), con, nodes);
-        TAILQ_INSERT_TAIL(&(ws->focus_head), con, focused);
     }
+    TAILQ_INSERT_TAIL(&(ws->focus_head), con, focused);
 
     /* Pretend the con was just opened with regards to size percent values.
      * Since the con is moved to a completely different con, the old value
      * does not make sense anyways. */
     con->percent = 0.0;
     con_fix_percent(ws);
+
+    con_fix_percent(old_parent);
+    CALL(old_parent, on_remove_child);
 }
 
 /*
@@ -98,7 +204,6 @@ static void attach_to_workspace(Con *con, Con *ws, direction_t direction) {
  *
  */
 static void move_to_output_directed(Con *con, direction_t direction) {
-    Con *old_ws = con_get_workspace(con);
     Output *current_output = get_output_for_con(con);
     Output *output = get_output_next(direction, current_output, CLOSEST_OUTPUT);
 
@@ -115,17 +220,26 @@ static void move_to_output_directed(Con *con, direction_t direction) {
         return;
     }
 
+    Con *old_ws = con_get_workspace(con);
+    const bool moves_focus = (focused == con);
     attach_to_workspace(con, ws, direction);
-
-    /* fix the focus stack */
-    con_focus(con);
+    if (moves_focus) {
+        /* workspace_show will not correctly update the active workspace because
+         * the focused container, con, is now a child of ws. To work around this
+         * and still produce the correct workspace focus events (see
+         * 517-regress-move-direction-ipc.t) we need to temporarily set focused
+         * to the old workspace. */
+        focused = old_ws;
+        workspace_show(ws);
+        con_focus(con);
+    }
 
     /* force re-painting the indicators */
     FREE(con->deco_render_params);
 
     tree_flatten(croot);
-
-    ipc_send_workspace_event("focus", ws, old_ws);
+    ipc_send_window_event("move", con);
+    ewmh_update_wm_desktop();
 }
 
 /*
@@ -146,13 +260,19 @@ void tree_move(Con *con, int direction) {
         return;
     }
 
-    if (con->parent->type == CT_WORKSPACE && con_num_children(con->parent) == 1) {
+    if (con->fullscreen_mode == CF_GLOBAL) {
+        DLOG("Not moving fullscreen global container\n");
+        return;
+    }
+
+    if ((con->fullscreen_mode == CF_OUTPUT) ||
+        (con->parent->type == CT_WORKSPACE && con_num_children(con->parent) == 1)) {
         /* This is the only con on this workspace */
         move_to_output_directed(con, direction);
         return;
     }
 
-    orientation_t o = (direction == D_LEFT || direction == D_RIGHT ? HORIZ : VERT);
+    orientation_t o = orientation_from_direction(direction);
 
     Con *same_orientation = con_parent_with_orientation(con, o);
     /* The do {} while is used to 'restart' at this point with a different
@@ -179,9 +299,10 @@ void tree_move(Con *con, int direction) {
 
         /* easy case: the move is within this container */
         if (same_orientation == con->parent) {
-            DLOG("We are in the same container\n");
-            Con *swap;
-            if ((swap = (direction == D_LEFT || direction == D_UP ? TAILQ_PREV(con, nodes_head, nodes) : TAILQ_NEXT(con, nodes)))) {
+            Con *swap = (direction == D_LEFT || direction == D_UP)
+                            ? TAILQ_PREV(con, nodes_head, nodes)
+                            : TAILQ_NEXT(con, nodes);
+            if (swap) {
                 if (!con_is_leaf(swap)) {
                     DLOG("Moving into our bordering branch\n");
                     target = con_descend_direction(swap, direction);
@@ -193,26 +314,22 @@ void tree_move(Con *con, int direction) {
                     insert_con_into(con, target, position);
                     goto end;
                 }
-                if (direction == D_LEFT || direction == D_UP)
+
+                DLOG("Swapping with sibling.\n");
+                if (direction == D_LEFT || direction == D_UP) {
                     TAILQ_SWAP(swap, con, &(swap->parent->nodes_head), nodes);
-                else
+                } else {
                     TAILQ_SWAP(con, swap, &(swap->parent->nodes_head), nodes);
+                }
 
-                TAILQ_REMOVE(&(con->parent->focus_head), con, focused);
-                TAILQ_INSERT_HEAD(&(swap->parent->focus_head), con, focused);
-
-                DLOG("Swapped.\n");
                 ipc_send_window_event("move", con);
-                ewmh_update_wm_desktop();
                 return;
             }
 
             if (con->parent == con_get_workspace(con)) {
-                /*  If we couldn't find a place to move it on this workspace,
-                 *  try to move it to a workspace on a different output */
+                /* If we couldn't find a place to move it on this workspace, try
+                 * to move it to a workspace on a different output */
                 move_to_output_directed(con, direction);
-                ipc_send_window_event("move", con);
-                ewmh_update_wm_desktop();
                 return;
             }
 
@@ -257,6 +374,7 @@ void tree_move(Con *con, int direction) {
          * and move it to the next output. */
         DLOG("Grandparent is workspace\n");
         move_to_output_directed(con, direction);
+        return;
     } else {
         DLOG("Moving into container above\n");
         position = (direction == D_UP || direction == D_LEFT ? BEFORE : AFTER);
@@ -264,11 +382,6 @@ void tree_move(Con *con, int direction) {
     }
 
 end:
-    /* We need to call con_focus() to fix the focus stack "above" the container
-     * we just inserted the focused container into (otherwise, the parent
-     * container(s) would still point to the old container(s)). */
-    con_focus(con);
-
     /* force re-painting the indicators */
     FREE(con->deco_render_params);
 
